## $Id: productoptions.py 12343 2014-12-30 12:52:40Z henrik $
##
## Copyright (C) 2012 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""Components with interfaces representing and aggregating product options.
"""
import grok
from decimal import Decimal
from zope import schema
from zope.schema.interfaces import IObject
from zope.interface import Attribute, Interface
from zope.formlib.interfaces import IInputWidget, IDisplayWidget
from zope.publisher.interfaces.browser import IBrowserRequest
from zope.schema.fieldproperty import FieldProperty
from zope.schema import Object
from waeup.ikoba.interfaces import SimpleIkobaVocabulary
from waeup.ikoba.interfaces import MessageFactory as _
from waeup.ikoba.widgets.objectwidget import (
    IkobaObjectWidget, IkobaObjectDisplayWidget
    )
from waeup.ikoba.payments.currencies import ISO_4217_CURRENCIES_VOCAB as currencies

#currencies = SimpleIkobaVocabulary(
#    (_('NGN'),'NGN'),
#    (_('USD'),'USD'),
#    (_('EUR'),'EUR'),
#    )

class IProductOption(Interface):
    """A product option entry.

    """
    title = schema.TextLine(
        title = _(u'Title'),
        required = True,
        )
    fee = schema.Decimal(
        title = _(u'Fee'),
        required = False,
        )
    currency = schema.Choice(
        title = _(u'Currency'),
        vocabulary = currencies,
        required = False,
        )

class IProductOptionField(IObject):
    """A zope.schema-like field for usage in interfaces.

    Marker interface to distuingish product options from ordinary
    object fields. Needed for registration of widgets.
    """

class ProductOption(grok.Model):
    """A product option entry contains a title, fee and currency.
    """
    grok.implements(IProductOption)
    title = FieldProperty(IProductOption['title'])
    fee = FieldProperty(IProductOption['fee'])
    currency = FieldProperty(IProductOption['currency'])

    def __init__(self, title=None, fee=None, currency=None):
        super(ProductOption, self).__init__()
        if title is not None:
            self.title = title
        if fee is not None:
            self.fee = fee
        if currency is not None:
            self.currency = currency
        return

    def to_string(self):
        """A string representation that can be used in exports.

        Returned is a unicode string of format ``(u'<TITLE>',u'<FEE>',u'<CURR>')``.
        """
        string = u"(u'%s', u'%s', u'%s')" % (self.title, self.fee, self.currency)
        string = string.replace("u'None'", "None")
        return string

    @classmethod
    def from_string(cls, string):
        """Create new ProductOption instance based on `string`.

        The string is expected to be in format as delivered by
        meth:`to_string`.

        This is a classmethod. This means, you normally will call::

          ProductOption.from_string(mystring)

        i.e. use the `ProductOption` class, not an instance thereof.
        """
        string = string.replace("u''", "None")
        title, fee, currency = eval(string)
        try:
            fee = Decimal(fee)
            title = unicode(title)
            currency = unicode(currency)
        except TypeError:
            pass
        return cls(title, fee, currency)

class ProductOptionField(Object):
    """A zope.schema-like field for usage in interfaces.

    If you want to define an interface containing product options, you
    can do so like this::

      class IMyInterface(Interface):
          my_option_entry = ProductOptionField()

    Default widgets are registered to render product option fields.
    """
    grok.implements(IProductOptionField)

    def __init__(self, **kw):
        super(ProductOptionField, self).__init__(IProductOption, **kw)
        return

# register IkobaObjectWidgets as default widgets for IProductOptionFields
@grok.adapter(IProductOptionField, IBrowserRequest)
@grok.implementer(IInputWidget)
def option_input_widget(obj, req):
    return IkobaObjectWidget(obj, req, ProductOption)

# register a display widget for IProductOptionFields
@grok.adapter(IProductOptionField, IBrowserRequest)
@grok.implementer(IDisplayWidget)
def option_display_widget(obj, req):
    return IkobaObjectDisplayWidget(obj, req, ProductOption)
